/***
 * Neuroph  http://neuroph.sourceforge.net
 * Copyright by Neuroph Project (C) 2008 
 *
 * This file is part of Neuroph framework.
 *
 * Neuroph is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Neuroph is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Neuroph. If not, see <http://www.gnu.org/licenses/>.
 */
package org.neuroph.contrib.tutorials.timeseries;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

/**
 * Very simple basic class which wraps the data generation process for the Time-Series
 * Prediction tutorial. Internally handles writing the data out to a system file. Also
 * uses a buffer to simulate stepping along the time series data.
 *
 * @author Laura Ellen Carter-Greaves
 */

public class GenerateSet {

	private String filename;
	private boolean simple;
	private int sampleEvery;
	private int memorySize;
	
		
	public GenerateSet(String filename, boolean simple, int sampleEvery, int memory) {
		super();
		this.filename = filename;
		this.simple = simple;
		this.sampleEvery = sampleEvery;
		this.memorySize = memory;
	}

	public void doIt() throws IOException {
		if(simple){
			doItSimple();
		}
		else{
			doItSupposition();
		}
	}
	
	private void doItSimple() throws IOException {
		
		double buffer[]=new double[memorySize+1]; //the extra one is for the output value
		int bufferSize=buffer.length;
		int bufferFull=buffer.length;
		
		GenerateSineWave gsw=new GenerateSineWave(1,1);
		
		BufferedWriter out=new BufferedWriter(new FileWriter(filename));
		
		for(int samples=0;samples<1000;samples++){
			
			//run through next samples so we take samples EVERY 'sampleEvery' steps
			for(int j=0;j<sampleEvery;j++){
				gsw.getNextSample();
			}
			
			//shift buffer down by one
			for(int i=1;i<bufferSize;i++){
				buffer[i-1]=buffer[i];
			}
			
			//load latest into end of buffer
			buffer[bufferSize-1]=gsw.getNextSample();
			
			//check to see when buffer fills up
			bufferFull--;
			if(bufferFull<=0){ //if buffer full write out the buffer
				for(int i=0;i<bufferSize;i++){
					out.write(buffer[i]+"\t");
				}
				out.write("\n");
			}
		}
		
		out.flush();
		out.close();
	}
	
	private void doItSupposition() throws IOException{
		
		BufferedWriter out=new BufferedWriter(new FileWriter(filename));
		
		double buffer[]=new double[memorySize+1]; //the extra one is for the output value
		int bufferSize=buffer.length;
		int bufferFull=buffer.length;
		
		GenerateSineWave gsw1=new GenerateSineWave(1,0.5);
		GenerateSineWave gsw2=new GenerateSineWave(4,0.5);
		
		for(int samples=0;samples<1000;samples++){
			
			//run through next samples so we take samples EVERY
			for(int j=0;j<sampleEvery;j++){
				gsw1.getNextSample();
				gsw2.getNextSample();
			}
			
			//shift buffer down by one
			for(int i=1;i<bufferSize;i++){
				buffer[i-1]=buffer[i];
			}
			
			//load latest into end of buffer
			buffer[bufferSize-1]=gsw1.getNextSample()+gsw2.getNextSample();
			
			//check to see when buffer fills up
			bufferFull--;
			if(bufferFull<=0){ //if buffer full write out the buffer
				for(int i=0;i<bufferSize;i++){
					out.write(buffer[i]+"\t");
				}
				out.write("\n");
			}
		}
		
		out.flush();
		out.close();
	}
	
}
